@@ -9,6 +9,7 @@
 #include "version.h"
 #include "fceu.h"
 #include "driver.h"
+#include "utils/memory.h"
 
 static char *aboutString = 0;
 
@@ -16,18 +17,19 @@ static char *aboutString = 0;
 char *FCEUI_GetAboutString() {
 	const char *aboutTemplate = 
 	FCEU_NAME_AND_VERSION "\n\n\
-Authors:\n\
-zeromus, adelikat,\n\n\
-Contributers:\n\
-Acmlm,CaH4e3\n\
-DWEdit,QFox\n\
-qeed,Shinydoofy,ugetab\n\
+Administrators:\n\
+zeromus, adelikat\n\n\
+Current Contributors:\n\
+punkrockguy318 (Lukas Sabota)\n\
+Plombo (Bryan Cain)\n\
+qeed, QFox, Shinydoofy, ugetab\n\
+CaH4e3, gocha, Acmlm, DWEdit\n\
 \n\
-FCEUX 2.0\n\
+FCEUX 2.0:\n\
 mz, nitsujrehtona, Lukas Sabota,\n\
 SP, Ugly Joe\n\
-\n\n\
-Previous versions:\n\n\
+\n\
+Previous versions:\n\
 FCE - Bero\n\
 FCEU - Xodnizel\n\
 FCEU XD - Bbitmaster & Parasyte\n\
@@ -36,6 +38,10 @@ FCEU MM - CaH4e3\n\
 FCEU TAS - blip & nitsuja\n\
 FCEU TAS+ - Luke Gustafson\n\
 \n\
+FCEUX is dedicated to the fallen heroes\n\
+of NES emulation. In Memoriam --\n\
+ugetab\n\
+\n\
 "__TIME__" "__DATE__"\n";
 
 	if(aboutString) return aboutString;
@@ -43,7 +49,9 @@ FCEU TAS+ - Luke Gustafson\n\
 	const char *compilerString = FCEUD_GetCompilerString();
 
 	//allocate the string and concatenate the template with the compiler string
-	aboutString = (char*)malloc(strlen(aboutTemplate) + strlen(compilerString) + 1);
-	sprintf(aboutString,"%s%s",aboutTemplate,compilerString);
+	if (!(aboutString = (char*)FCEU_dmalloc(strlen(aboutTemplate) + strlen(compilerString) + 1)))
+        return NULL;
+	
+    sprintf(aboutString,"%s%s",aboutTemplate,compilerString);
 	return aboutString;
 }
