@@ -19,7 +19,11 @@
 
 #ifndef FCEUX_GUI_H
 #define FCEUX_GUI_H
-#define GTK_LITE
+#define GTK
+#include <gtk/gtk.h>
+extern GtkWidget* MainWindow;
+extern GtkWidget* socket;
+extern GtkRadioAction* stateSlot;
 int InitGTKSubsystem(int argc, char** argv);
 void pushOutputToGTK(const char* str);
 void showGui(bool b);
