@@ -31,6 +31,8 @@
 #include "../../fceu.h"
 #include "../../version.h"
 
+#include "../../utils/memory.h"
+
 #include "sdl-icon.h"
 #include "dface.h"
 
@@ -43,6 +45,7 @@
 
 #ifdef _GTK
 #include "gui.h"
+#include <gdk/gdkx.h>
 #endif
 
 // GLOBALS
@@ -67,6 +70,8 @@ static int s_eefx;
 static int s_clipSides;
 static int s_fullscreen;
 static int noframe;
+static int s_nativeWidth = -1;
+static int s_nativeHeight = -1;
 
 #define NWIDTH	(256 - (s_clipSides ? 16 : 0))
 #define NOFFSET	(s_clipSides ? 8 : 0)
@@ -203,6 +208,14 @@ InitVideo(FCEUGI *gi)
         flags |= SDL_HWSURFACE;
     }
     
+    // get the monitor's current resolution if we do not already have it
+    if(s_nativeWidth < 0) {
+    	s_nativeWidth = vinf->current_w;
+    }
+    if(s_nativeHeight < 0) {
+    	s_nativeHeight = vinf->current_h;
+    }
+    
     // check if we are rendering fullscreen
     if(s_fullscreen) {
         flags |= SDL_FULLSCREEN;
@@ -310,8 +323,14 @@ InitVideo(FCEUGI *gi)
                 return -1;
             }
 
-
+#ifdef OPENGL
+        s_screen = SDL_SetVideoMode(s_useOpenGL ? s_nativeWidth : xres,
+        							s_useOpenGL ? s_nativeHeight : yres,
+        							desbpp, flags);
+#else
         s_screen = SDL_SetVideoMode(xres, yres, desbpp, flags);
+#endif
+
         if(!s_screen) {
             FCEUD_PrintError(SDL_GetError());
             return -1;
@@ -355,6 +374,29 @@ InitVideo(FCEUGI *gi)
 			return -1;
 		}
 #endif
+
+#if defined(_GTK) && defined(SDL_VIDEO_DRIVER_X11)
+		int noGui;
+		g_config->getOption("SDL.NoGUI", &noGui);
+		if(noGui == 0)
+		{
+			while (gtk_events_pending())
+				gtk_main_iteration_do(FALSE);
+        
+			char SDL_windowhack[128];
+			sprintf(SDL_windowhack, "SDL_WINDOWID=%u", (unsigned int)GDK_WINDOW_XWINDOW(gtk_widget_get_window(socket)));
+			SDL_putenv(SDL_windowhack);
+        
+			// init SDL video
+			if (SDL_WasInit(SDL_INIT_VIDEO))
+				SDL_QuitSubSystem(SDL_INIT_VIDEO);
+			if ( SDL_InitSubSystem(SDL_INIT_VIDEO) < 0 )
+			{
+				fprintf(stderr, "Couldn't init SDL video: %s\n", SDL_GetError());
+				gtk_main_quit();
+			}
+		}
+#endif
         
         s_screen = SDL_SetVideoMode((int)(NWIDTH * s_exs),
                                   (int)(s_tlines * s_eys),
@@ -363,6 +405,12 @@ InitVideo(FCEUGI *gi)
             FCEUD_PrintError(SDL_GetError());
             return -1;
         }
+
+#ifdef _GTK
+        GtkRequisition req;
+        gtk_widget_size_request(GTK_WIDGET(MainWindow), &req);
+        gtk_window_resize(GTK_WINDOW(MainWindow), req.width, req.height);
+#endif
     }
     s_curbpp = s_screen->format->BitsPerPixel;
     if(!s_screen) {
@@ -446,6 +494,11 @@ InitVideo(FCEUGI *gi)
 void
 ToggleFS()
 {
+    // pause while we we are making the switch
+    bool paused = FCEUI_EmulationPaused();
+    if(!paused)
+        FCEUI_ToggleEmulationPause();
+
     int error, fullscreen = s_fullscreen;
 
     // shut down the current video system
@@ -466,6 +519,9 @@ ToggleFS()
         g_config->setOption("SDL.Fullscreen", fullscreen);
         InitVideo(GameInfo);
     }
+    // if we paused to make the switch; unpause
+    if(!paused)
+        FCEUI_ToggleEmulationPause();
 }
 
 static SDL_Color s_psdl[256];
@@ -650,7 +706,7 @@ BlitScreen(uint8 *XBuf)
    if(!result || resultsize != width*height*3*2)
    {
        if(result) free(result);
-       result = (unsigned char*) malloc(resultsize = width*height*3*2);
+       result = (unsigned char*) FCEU_dmalloc(resultsize = width*height*3*2);
    }
    switch(s_curbpp)
    {
