@@ -23,10 +23,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef _USE_SHARED_MEMORY_
-#include <windows.h>
-#endif
-
 #include "types.h"
 #include "x6502.h"
 #include "fceu.h"
@@ -50,14 +46,11 @@ extern SFORMAT FCEUVSUNI_STATEINFO[];
 
 //mbg merge 6/29/06 - these need to be global
 uint8 *trainerpoo=0;
-uint8 *ROM=NULL;
-uint8 *VROM=NULL;
+uint8 *ROM = NULL;
+uint8 *VROM = NULL;
 iNES_HEADER head ;
 
 
-#ifdef _USE_SHARED_MEMORY_
-HANDLE mapROM, mapVROM;
-#endif
 
 static CartInfo iNESCart;
 
@@ -125,39 +118,8 @@ void iNESGI(GI h) //bbit edited: removed static keyword
 			FCEU_SaveGameSave(&iNESCart);
 
 			if(iNESCart.Close) iNESCart.Close();
-#ifdef _USE_SHARED_MEMORY_
-			if(ROM)
-		 {
-			 if(mapROM)
-			 {
-				 UnmapViewOfFile(mapROM);
-				 CloseHandle(mapROM);
-				 ROM=0;
-			 }
-			 else
-			 {
-				 free(ROM);
-				 ROM = NULL;
-			 }
-		 }
-			if(VROM)
-		 {
-			 if(mapVROM)
-			 {
-				 UnmapViewOfFile(mapVROM);
-				 CloseHandle(mapVROM);
-				 VROM=0;
-			 }
-			 else
-			 {
-				 free(VROM);
-				 VROM = NULL;
-			 }
-		 }
-#else
-			if(ROM) {free(ROM);ROM=0;}
-			if(VROM) {free(VROM);VROM=0;}
-#endif
+			if(ROM) {free(ROM); ROM = NULL;}
+			if(VROM) {free(VROM); VROM = NULL;}
 			if(MapClose) MapClose();
 			if(trainerpoo) {FCEU_gfree(trainerpoo);trainerpoo=0;}
 		}
@@ -389,7 +351,7 @@ static void CheckHInfo(void)
 		0x1b084107d0878bd0LL,    /* Startropics 2*/
 
 		0x836c0ff4f3e06e45LL,    /* Zelda 2 */
-	    
+
 		0x82000965f04a71bbLL,    /* Mirai Shinwa Jarvas */
 
 		0      /* Abandon all hope if the game has 0 in the lower 64-bits of its MD5 hash */
@@ -421,20 +383,8 @@ static void CheckHInfo(void)
 				if(moo[x].mapper&0x800 && VROM_size)
 				{
 					VROM_size=0;
-#ifdef _USE_SHARED_MEMORY_
-					if(mapVROM)
-					{
-						UnmapViewOfFile(mapVROM);
-						CloseHandle(mapVROM);
-					}
-					else
-					{
-						free(VROM);
-					}
-#else
 					free(VROM);
-#endif
-					VROM=0;
+					VROM = NULL;
 					tofix|=8;
 				}
 				if(MapperNo!=(moo[x].mapper&0xFF))
@@ -656,7 +606,8 @@ static BMAPPINGLocal bmap[] = {
 	{"",					215, Mapper215_Init},
 	{"",					216, Mapper216_Init},
 	{"",					217, Mapper217_Init},
-    {"UNLA9746",			219, UNLA9746_Init},
+	{"UNLA9746",			219, UNLA9746_Init},
+	{"OneBus",	 		    220, UNLOneBus_Init},
 
 //    {220, BMCFK23C_Init},
 //    {220, UNL3DBlock_Init},
@@ -736,56 +687,17 @@ int iNESLoad(const char *name, FCEUFILE *fp, int OverwriteVidMode)
 
 	if(head.ROM_type&8) Mirroring=2;
 
-#ifdef _USE_SHARED_MEMORY_
-	mapROM = CreateFileMapping((HANDLE)0xFFFFFFFF,NULL,PAGE_READWRITE, 0, ROM_size<<14,"fceu.ROM");
-	if(mapROM == NULL || GetLastError() == ERROR_ALREADY_EXISTS)
-	{
-		mapROM = NULL;
-		CloseHandle(mapROM);
-
-		if(!(ROM=(uint8 *)FCEU_malloc(ROM_size<<14)))
-			return 0;
-
-		if(VROM_size)
-		{
-			if(!(VROM=(uint8 *)FCEU_malloc(VROM_size<<13)))
-			{
-				free(ROM);
-				ROM = NULL;
-				return 0;
-			}
-		}
-	}
-	else
-	{
-		ROM   = (uint8 *)MapViewOfFile(mapROM, FILE_MAP_WRITE, 0, 0, 0);
-		if( !ROM ) return 0;
-
-		if(VROM_size)
-		{
-			mapVROM = CreateFileMapping((HANDLE)0xFFFFFFFF,NULL,PAGE_READWRITE, 0, VROM_size<<13,"fceu.VROM");
-			VROM   = (uint8 *)MapViewOfFile(mapVROM, FILE_MAP_WRITE, 0, 0, 0);
-			if( !VROM )
-			{
-				UnmapViewOfFile(mapROM);
-				CloseHandle(mapROM);
-				return 0;
-			}
-		}
-	}
-#else
-	if(!(ROM=(uint8 *)FCEU_malloc(ROM_size<<14)))
-		return 0;
+	if((ROM = (uint8 *)FCEU_malloc(ROM_size<<14)) == NULL) return 0;
 
 	if(VROM_size)
 	{
-		if(!(VROM=(uint8 *)FCEU_malloc(VROM_size<<13)))
+		if((VROM = (uint8 *)FCEU_malloc(VROM_size<<13)) == NULL)
 		{
 			free(ROM);
+			ROM = NULL;
 			return 0;
 		}
 	}
-#endif
 	memset(ROM,0xFF,ROM_size<<14);
 	if(VROM_size) memset(VROM,0xFF,VROM_size<<13);
 	if(head.ROM_type&4)   /* Trainer */
@@ -890,7 +802,7 @@ int iNESLoad(const char *name, FCEUFILE *fp, int OverwriteVidMode)
 	FCEU_LoadGameSave(&iNESCart);
 
 	strcpy(LoadedRomFName,name); //bbit edited: line added
-	
+
 	// Extract Filename only. Should account for Windows/Unix this way.
 	if (strrchr(name, '/')) {
 	name = strrchr(name, '/') + 1;
@@ -1465,7 +1377,7 @@ static int NewiNES_Init(int num)
 
 	if(GameInfo->type == GIT_VSUNI)
 		AddExState(FCEUVSUNI_STATEINFO, ~0, 0, 0);
-		
+
 	while(tmp->init)
 	{
 		if(num==tmp->number)
@@ -1482,21 +1394,7 @@ static int NewiNES_Init(int num)
 				{
 					CHRRAMSize=8192;
 				}
-#ifdef _USE_SHARED_MEMORY_
-				mapVROM = CreateFileMapping((HANDLE)0xFFFFFFFF,NULL,PAGE_READWRITE, 0, CHRRAMSize,"fceu.VROM");
-				if(mapVROM == NULL || GetLastError() == ERROR_ALREADY_EXISTS)
-				{
-					CloseHandle(mapVROM);
-					mapVROM = NULL;
-					VROM=(uint8 *)malloc(CHRRAMSize);
-				}
-				else
-				{
-					VROM   = (uint8 *)MapViewOfFile(mapVROM, FILE_MAP_WRITE, 0, 0, 0);
-				}
-#else
-				VROM=(uint8 *)malloc(CHRRAMSize);
-#endif
+				if((VROM = (uint8 *)FCEU_dmalloc(CHRRAMSize)) == NULL) return 0;
 				UNIFchrrama=VROM;
 				SetupCartCHRMapping(0,VROM,CHRRAMSize,1);
 				AddExState(VROM,CHRRAMSize, 0, "CHRR");
@@ -1504,9 +1402,9 @@ static int NewiNES_Init(int num)
 			if(head.ROM_type&8)
 				AddExState(ExtraNTARAM, 2048, 0, "EXNR");
 			tmp->init(&iNESCart);
-			return(1);
+			return 1;
 		}
 		tmp++;
 	}
-	return(0);
+	return 0;
 }
