@@ -1,19 +1,19 @@
 /* FCE Ultra - NES/Famicom Emulator
-* 
+*
 * Copyright notice for this file:
 *  Copyright (C) 1998 BERO
 *  Copyright (C) 2003 Xodnizel
-*  
+*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
-* the Free Software Foundation; either version 2 of the License, or   
+* the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
-* 
+*
 * This program is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of 
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
-* 
+*
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -42,7 +42,7 @@
 
 
 #define VBlankON  (PPU[0]&0x80)   //Generate VBlank NMI
-#define Sprite16  (PPU[0]&0x20)   //Sprites 8x16/8x8 
+#define Sprite16  (PPU[0]&0x20)   //Sprites 8x16/8x8
 #define BGAdrHI   (PPU[0]&0x10)   //BG pattern adr $0000/$1000
 #define SpAdrHI   (PPU[0]&0x08)   //Sprite pattern adr $0000/$1000
 #define INC32     (PPU[0]&0x04)   //auto increment 1/32
@@ -57,7 +57,7 @@
 
 #define PPU_status      (PPU[2])
 
-#define Pal     (PALRAM)   
+#define Pal     (PALRAM)
 
 static void FetchSpriteData(void);
 static void RefreshLine(int lastpixel);
@@ -155,7 +155,7 @@ struct PPUREGS {
 
 	//temp unlatched regs (need savestating, can be written to at any time)
     uint32 _fv, _v, _h, _vt, _ht;
-	
+
 	//other regs that need savestating
 	uint32 fh;//3 (horz scroll)
 	uint32 s;//1 ($2000 bit 4: "Background pattern table address (0: $0000; 1: $1000)")
@@ -172,11 +172,11 @@ struct PPUREGS {
 		fv = v = h = vt = ht = 0;
 		fh = par = s = 0;
 		_fv = _v = _h = _vt = _ht = 0;
-		status.cycle = 0; 
+		status.cycle = 0;
 		status.end_cycle = 341;
-		status.sl = 241; 
+		status.sl = 241;
 	}
-        
+
 	void install_latches() {
 		fv = _fv;
 		v = _v;
@@ -196,8 +196,8 @@ struct PPUREGS {
 	}
 
 	void increment_hsc() {
-		//The first one, the horizontal scroll counter, consists of 6 bits, and is 
-		//made up by daisy-chaining the HT counter to the H counter. The HT counter is 
+		//The first one, the horizontal scroll counter, consists of 6 bits, and is
+		//made up by daisy-chaining the HT counter to the H counter. The HT counter is
 		//then clocked every 8 pixel dot clocks (or every 8/3 CPU clock cycles).
 		ht++;
 		h += (ht>>5);
@@ -222,12 +222,12 @@ struct PPUREGS {
 	uint32 get_2007access() {
 		return ((fv&3)<<0xC) | (v<<0xB) | (h<<0xA) | (vt<<5) | ht;
 	}
-	
-	//The PPU has an internal 4-position, 2-bit shifter, which it uses for 
-	//obtaining the 2-bit palette select data during an attribute table byte 
-	//fetch. To represent how this data is shifted in the diagram, letters a..c 
-	//are used in the diagram to represent the right-shift position amount to 
-	//apply to the data read from the attribute data (a is always 0). This is why 
+
+	//The PPU has an internal 4-position, 2-bit shifter, which it uses for
+	//obtaining the 2-bit palette select data during an attribute table byte
+	//fetch. To represent how this data is shifted in the diagram, letters a..c
+	//are used in the diagram to represent the right-shift position amount to
+	//apply to the data read from the attribute data (a is always 0). This is why
 	//you only see bits 0 and 1 used off the read attribute data in the diagram.
 	uint32 get_atread() {
 		return 0x2000 | (v<<0xB) | (h<<0xA) | 0x3C0 | ((vt&0x1C)<<1) | ((ht&0x1C)>>2);
@@ -239,15 +239,15 @@ struct PPUREGS {
 	}
 
 	void increment2007(bool by32) {
-	
-		//If the VRAM address increment bit (2000.2) is clear (inc. amt. = 1), all the 
-		//scroll counters are daisy-chained (in the order of HT, VT, H, V, FV) so that 
-		//the carry out of each counter controls the next counter's clock rate. The 
-		//result is that all 5 counters function as a single 15-bit one. Any access to 
+
+		//If the VRAM address increment bit (2000.2) is clear (inc. amt. = 1), all the
+		//scroll counters are daisy-chained (in the order of HT, VT, H, V, FV) so that
+		//the carry out of each counter controls the next counter's clock rate. The
+		//result is that all 5 counters function as a single 15-bit one. Any access to
 		//2007 clocks the HT counter here.
 		//
-		//If the VRAM address increment bit is set (inc. amt. = 32), the only 
-		//difference is that the HT counter is no longer being clocked, and the VT 
+		//If the VRAM address increment bit is set (inc. amt. = 32), the only
+		//difference is that the HT counter is no longer being clocked, and the VT
 		//counter is now being clocked by access to 2007.
 		if(by32) {
 			vt++;
@@ -302,7 +302,7 @@ static void makeppulut(void)
 			//    printf("%08x\n",ppulut3[xo|(cc<<3)]);
 		}
 	}
-} 
+}
 
 static int ppudead=1;
 static int kook=0;
@@ -318,16 +318,16 @@ uint32 MMC5HackVROMMask=0;
 uint8 *MMC5HackExNTARAMPtr=0;
 uint8 *MMC5HackVROMPTR=0;
 uint8 MMC5HackCHRMode=0;
-uint8 MMC5HackSPMode=0;   
+uint8 MMC5HackSPMode=0;
 uint8 MMC50x5130=0;
-uint8 MMC5HackSPScroll=0; 
+uint8 MMC5HackSPScroll=0;
 uint8 MMC5HackSPPage=0;
 
 
 uint8 VRAMBuffer=0,PPUGenLatch=0;
 uint8 *vnapage[4];
-uint8 PPUNTARAM=0;  
-uint8 PPUCHRRAM=0;  
+uint8 PPUNTARAM=0;
+uint8 PPUCHRRAM=0;
 
 //Color deemphasis emulation.  Joy...
 static uint8 deemp=0;
@@ -341,10 +341,11 @@ uint8 XOffset=0;
 
 uint32 TempAddr=0,RefreshAddr=0;
 
-static int maxsprites=8;  
+static int maxsprites=8;
 
 //scanline is equal to the current visible scanline we're on.
 int scanline;
+int g_rasterpos;
 static uint32 scanlines_per_frame;
 
 uint8 PPU[4];
@@ -365,7 +366,7 @@ uint8 * MMC5BGVRAMADR(uint32 V) {
 		extern uint8 mmc5ABMode;                /* A=0, B=1 */
 		if(mmc5ABMode==0)
 			return MMC5SPRVRAMADR(V);
-		else 
+		else
 			return &MMC5BGVPage[(V)>>10][(V)];
 	} else return &MMC5BGVPage[(V)>>10][(V)];
 }
@@ -409,7 +410,7 @@ inline void FFCEUX_PPUWrite_Default(uint32 A, uint8 V) {
     {
         if(PPUCHRRAM&(1<<(tmp>>10)))
             VPage[tmp>>10][tmp]=V;
-    }   
+    }
     else if (tmp<0x3F00)
     {
         if(PPUNTARAM&(1<<((tmp&0xF00)>>10)))
@@ -420,7 +421,7 @@ inline void FFCEUX_PPUWrite_Default(uint32 A, uint8 V) {
         if (!(tmp & 3))
         {
             if (!(tmp & 0xC))
-                  PALRAM[0x00] = PALRAM[0x04] = 
+                  PALRAM[0x00] = PALRAM[0x04] =
                   PALRAM[0x08] = PALRAM[0x0C] = V & 0x3F;
             else
                 UPALRAM[((tmp & 0xC) >> 2) - 1] = V & 0x3F;
@@ -440,7 +441,7 @@ uint8 FASTCALL FFCEUX_PPURead_Default(uint32 A) {
 		return VPage[tmp>>10][tmp];
 	}
 	else if (tmp < 0x3F00)
-	{   
+	{
 		return vnapage[(tmp>>10)&0x3][tmp&0x3FF];
 	}
     else
@@ -455,7 +456,7 @@ uint8 FASTCALL FFCEUX_PPURead_Default(uint32 A) {
         }
         else
             ret = PALRAM[tmp & 0x1F];
-            
+
         if (GRAYSCALE)
             ret &= 0x30;
         return ret;
@@ -484,7 +485,7 @@ void ppu_getScroll(int &xpos, int &ypos)
 	{
 		xpos = ((RefreshAddr & 0x400) >> 2) | ((RefreshAddr & 0x1F) << 3) | XOffset;
 
-		ypos = ((RefreshAddr & 0x3E0) >> 2) | ((RefreshAddr & 0x7000) >> 12); 
+		ypos = ((RefreshAddr & 0x3E0) >> 2) | ((RefreshAddr & 0x7000) >> 12);
 			if(RefreshAddr & 0x800) ypos += 240;
 	}
 }
@@ -498,7 +499,7 @@ static DECLFR(A2002)
 		//i think we should only reset the state machine for 2005/2006
 		//ppur.clear_latches();
 	}
-	
+
 	uint8 ret;
 
 	FCEUPPU_LineUpdate();
@@ -528,9 +529,9 @@ static DECLFR(A2004)
              * to 0xFF */
             if (ppur.status.cycle < 64)
                 return spr_read.ret = 0xFF;
-            else 
+            else
             {
-                for (int i = spr_read.last; 
+                for (int i = spr_read.last;
                      i != ppur.status.cycle; ++i)
                 {
                     if (i < 256)
@@ -539,11 +540,11 @@ static DECLFR(A2004)
                         {
                             case 0:
                                 if (spr_read.count < 2)
-                                    spr_read.ret = (PPU[3] & 0xF8) 
+                                    spr_read.ret = (PPU[3] & 0xF8)
                                     + (spr_read.count << 2);
                                 else
                                     spr_read.ret = spr_read.count << 2;
-                                spr_read.found_pos[spr_read.found] = 
+                                spr_read.found_pos[spr_read.found] =
                                     spr_read.ret;
 
                                 spr_read.ret = SPRAM[spr_read.ret];
@@ -551,8 +552,8 @@ static DECLFR(A2004)
                                 if (i & 1) //odd cycle
                                 {
                                     //see if in range
-                                    if ( !((ppur.status.sl - 1 - 
-                                            spr_read.ret) 
+                                    if ( !((ppur.status.sl - 1 -
+                                            spr_read.ret)
                                             & ~(Sprite16 ? 0xF : 0x7)) )
 
                                     {
@@ -560,7 +561,7 @@ static DECLFR(A2004)
                                         spr_read.fetch = 1;
                                         spr_read.mode = 1;
                                     }
-                                    else 
+                                    else
                                     {
                                         if (++spr_read.count == 64)
                                         {
@@ -598,23 +599,23 @@ static DECLFR(A2004)
                                 }
 
                                 if (spr_read.count < 2)
-                                    spr_read.ret = (PPU[3] & 0xF8) 
+                                    spr_read.ret = (PPU[3] & 0xF8)
                                         + (spr_read.count << 2);
                                 else
                                     spr_read.ret = spr_read.count << 2;
 
-                                spr_read.ret = SPRAM[spr_read.ret | 
+                                spr_read.ret = SPRAM[spr_read.ret |
                                     spr_read.fetch];
                                 break;
-                            case 2: //8th sprite fetched 
-                                spr_read.ret = SPRAM[(spr_read.count << 2) 
+                            case 2: //8th sprite fetched
+                                spr_read.ret = SPRAM[(spr_read.count << 2)
                                     | spr_read.fetch];
                                 if (i & 1)
                                 {
-                                    if ( !((ppur.status.sl - 1 - 
+                                    if ( !((ppur.status.sl - 1 -
                                               SPRAM[((spr_read.count << 2)
-                                                    | spr_read.fetch)]) 
-                                            & ~((Sprite16) ? 0xF : 0x7)) ) 
+                                                    | spr_read.fetch)])
+                                            & ~((Sprite16) ? 0xF : 0x7)) )
                                     {
                                         spr_read.fetch = 1;
                                         spr_read.mode = 3;
@@ -626,20 +627,20 @@ static DECLFR(A2004)
                                             spr_read.count = 0;
                                             spr_read.mode = 4;
                                         }
-                                        spr_read.fetch = 
+                                        spr_read.fetch =
                                             (spr_read.fetch + 1) & 3;
                                     }
                                 }
                                 spr_read.ret = spr_read.count;
                                 break;
                             case 3: //9th sprite overflow detected
-                                spr_read.ret = SPRAM[spr_read.count 
+                                spr_read.ret = SPRAM[spr_read.count
                                                | spr_read.fetch];
                                 if (i & 1)
                                 {
                                     if (++spr_read.fetch == 4)
                                     {
-                                        spr_read.count = (spr_read.count 
+                                        spr_read.count = (spr_read.count
                                                 + 1) & 63;
                                         spr_read.mode = 4;
                                     }
@@ -647,7 +648,7 @@ static DECLFR(A2004)
                                 break;
                             case 4: //read OAM[n][0] until hblank
                                 if (i & 1)
-                                    spr_read.count = 
+                                    spr_read.count =
                                         (spr_read.count + 1) & 63;
                                 spr_read.fetch = 0;
                                 spr_read.ret = SPRAM[spr_read.count << 2];
@@ -664,13 +665,13 @@ static DECLFR(A2004)
                                 spr_read.ret = SPRAM[252];
                                 spr_read.num = 0;
                             }
-                            else 
+                            else
                                 spr_read.ret = 0xFF;
                         }
                         else if ((i & 7) < 4)
-                        { 
-                            spr_read.ret = 
-                                SPRAM[spr_read.found_pos[spr_read.ret] 
+                        {
+                            spr_read.ret =
+                                SPRAM[spr_read.found_pos[spr_read.ret]
                                       | spr_read.fetch++];
                             if (spr_read.fetch == 4)
                                 spr_read.fetch = 0;
@@ -679,7 +680,7 @@ static DECLFR(A2004)
                             spr_read.ret = SPRAM[spr_read.found_pos
                                                  [spr_read.ret | 3]];
                     }
-                    else 
+                    else
                     {
                         if (!spr_read.found)
                             spr_read.ret = SPRAM[252];
@@ -716,7 +717,7 @@ uint8 ret;
 FCEUPPU_LineUpdate();
 ret = SPRAM[PPU[3]];
 
-if(PPUSPL>=8) 
+if(PPUSPL>=8)
 {
 if(PPU[3]>=8)
 ret = SPRAM[PPU[3]];
@@ -743,7 +744,7 @@ static DECLFR(A2007)
 		RefreshAddr = ppur.get_2007access() & 0x3FFF;
         if ((RefreshAddr & 0x3F00) == 0x3F00)
         {
-            //if it is in the palette range bypass the 
+            //if it is in the palette range bypass the
             //delayed read, and what gets filled in the temp
             //buffer is the address - 0x1000, also
             //if grayscale is set then the return is AND with 0x30
@@ -782,7 +783,7 @@ static DECLFR(A2007)
 				VRAMBuffer=VPage[tmp>>10][tmp];
 			}
 			else if (tmp < 0x3F00)
-			{   
+			{
 				VRAMBuffer=vnapage[(tmp>>10)&0x3][tmp&0x3FF];
 			}
 		}
@@ -790,8 +791,31 @@ static DECLFR(A2007)
 		if(!fceuindbg)
 	#endif
 		{
-			if(INC32) RefreshAddr+=32;
-			else RefreshAddr++;
+			if((ScreenON || SpriteON) && (scanline < 240))
+			{
+				uint32 rad=RefreshAddr;
+
+				if((rad&0x7000)==0x7000)
+				{
+					rad^=0x7000;
+					if((rad&0x3E0)==0x3A0)
+						rad^=0xBA0;
+					else if((rad&0x3E0)==0x3e0)
+						rad^=0x3e0;
+					else
+						rad+=0x20;
+				}
+				else
+					rad+=0x1000;
+				RefreshAddr=rad;
+			}
+			else
+			{
+				if (INC32)
+					RefreshAddr+=32;
+				else
+					RefreshAddr++;
+			}
 			if(PPU_hook) PPU_hook(RefreshAddr&0x3fff);
 		}
 
@@ -854,17 +878,17 @@ static DECLFW(B2004)
         if ((PPU[3] & 3) == 2)
             V &= 0xE3;
         SPRAM[PPU[3]] = V;
-        PPU[3] = (PPU[3] + 1) & 0xFF; 
-    }  
+        PPU[3] = (PPU[3] + 1) & 0xFF;
+    }
     else
     {
-        if(PPUSPL>=8) 
+        if(PPUSPL>=8)
         {
             if(PPU[3]>=8)
                 SPRAM[PPU[3]]=V;
         }
         else
-        {   
+        {
             //printf("$%02x:$%02x\n",PPUSPL,V);
             SPRAM[PPUSPL]=V;
         }
@@ -881,13 +905,13 @@ static DECLFW(B2005)
 	if(!vtoggle)
 	{
 		tmp&=0xFFE0;
-		tmp|=V>>3;  
+		tmp|=V>>3;
 		XOffset=V&7;
 		ppur._ht = V>>3;
 		ppur.fh = V&7;
 	}
 	else
-	{   
+	{
 		tmp&=0x8C1F;
 		tmp|=((V&~0x7)<<2);
 		tmp|=(V&7)<<12;
@@ -895,7 +919,7 @@ static DECLFW(B2005)
 		ppur._fv = V&7;
 	}
 	TempAddr=tmp;
-	vtoggle^=1;  
+	vtoggle^=1;
 }
 
 
@@ -906,11 +930,11 @@ static DECLFW(B2006)
 
 
 	PPUGenLatch=V;
-	if(!vtoggle)  
+	if(!vtoggle)
 	{
 		TempAddr&=0x00FF;
 		TempAddr|=(V&0x3f)<<8;
-		
+
 		ppur._vt &= 0x07;
 		ppur._vt |= (V&0x3)<<3;
 		ppur._h = (V>>2)&1;
@@ -918,7 +942,7 @@ static DECLFW(B2006)
 		ppur._fv = (V>>4)&3;
 	}
  	else
-	{   
+	{
 		TempAddr&=0xFF00;
 		TempAddr|=V;
 
@@ -947,7 +971,7 @@ static DECLFW(B2007)
 		//printf("%04x ",RefreshAddr);
 		ppur.increment2007(INC32!=0);
 		RefreshAddr = ppur.get_2007access();
-	} 
+	}
 	else
 	{
 		//printf("%04x ",tmp);
@@ -963,7 +987,7 @@ static DECLFW(B2007)
 		{
 			if(PPUCHRRAM&(1<<(tmp>>10)))
 				VPage[tmp>>10][tmp]=V;
-		}   
+		}
 		else
 		{
 			if(PPUNTARAM&(1<<((tmp&0xF00)>>10)))
@@ -990,7 +1014,7 @@ static DECLFW(B4014)
 #define GETLASTPIXEL    (PAL?((timestamp*48-linestartts)/15) : ((timestamp*48-linestartts)>>4) )
 
 static uint8 *Pline,*Plinef;
-static int firsttile;  
+static int firsttile;
 int linestartts;	//no longer static so the debugger can see it
 static int tofix=0;
 
@@ -999,12 +1023,12 @@ static void ResetRL(uint8 *target)
 	memset(target,0xFF,256);
 	InputScanlineHook(0,0,0,0);
 	Plinef=target;
-	Pline=target; 
+	Pline=target;
 	firsttile=0;
 	linestartts=timestamp*48+X.count;
 	tofix=0;
 	FCEUPPU_LineUpdate();
-	tofix=1;  
+	tofix=1;
 }
 
 static uint8 sprlinebuf[256+8];
@@ -1019,7 +1043,7 @@ void FCEUPPU_LineUpdate(void)
 			int l=GETLASTPIXEL;
 			RefreshLine(l);
 		}
-} 
+}
 
 static bool rendersprites=true, renderbg=true;
 
@@ -1048,10 +1072,10 @@ void FCEUI_GetRenderPlanes(bool& sprites, bool& bg)
 //{
 //	uint8 *P=XBuf+16*256;
 //	int bgh;
-//	int y;  
-//	int X1; 
+//	int y;
+//	int X1;
 //	for(bgh=0;bgh<2;bgh++)
-//		for(y=0;y<16*8;y++)   
+//		for(y=0;y<16*8;y++)
 //			for(P=XBuf+bgh*128+(16+y)*256,X1=16;X1;X1--,P+=8)
 //			{
 //				uint8 *C;
@@ -1066,7 +1090,7 @@ void FCEUI_GetRenderPlanes(bool& sprites, bool& bg)
 //				cc=0;
 //				//#include "pputile.inc"
 //			}
-//} 
+//}
 
 static void CheckSpriteHit(int p);
 
@@ -1092,7 +1116,7 @@ static void CheckSpriteHit(int p)
 	for(x=sphitx;x<(sphitx+8) && x<l;x++)
 	{
 
-        if((sphitdata&(0x80>>(x-sphitx))) && !(Plinef[x]&64) && x < 255) 
+        if((sphitdata&(0x80>>(x-sphitx))) && !(Plinef[x]&64) && x < 255)
 		{
 			PPU_status|=0x40;
 			//printf("Ha:  %d, %d, Hita: %d, %d, %d, %d, %d\n",p,p&~7,scanline,GETLASTPIXEL-16,&Plinef[x],Pline,Pline-Plinef);
@@ -1103,12 +1127,12 @@ static void CheckSpriteHit(int p)
 			break;
 		}
 	}
-}   
+}
 
-//spork the world.  Any sprites on this line? Then this will be set to 1.  
+//spork the world.  Any sprites on this line? Then this will be set to 1.
 //Needed for zapper emulation and *gasp* sprite emulation.
-static int spork=0;     
-						
+static int spork=0;
+
 // lasttile is really "second to last tile."
 static void RefreshLine(int lastpixel)
 {
@@ -1169,7 +1193,7 @@ static void RefreshLine(int lastpixel)
 			tofix=0;
 		}
 
-		if((lastpixel-16)>=0) 
+		if((lastpixel-16)>=0)
 		{
 			InputScanlineHook(Plinef,spork?sprlinebuf:0,linestartts,lasttile*8-16);
 		}
@@ -1177,12 +1201,12 @@ static void RefreshLine(int lastpixel)
 	}
 
 	//Priority bits, needed for sprite emulation.
-	Pal[0]|=64; 
+	Pal[0]|=64;
 	Pal[4]|=64;
 	Pal[8]|=64;
 	Pal[0xC]|=64;
 
-	//This high-level graphics MMC5 emulation code was written for MMC5 carts in "CL" mode.  
+	//This high-level graphics MMC5 emulation code was written for MMC5 carts in "CL" mode.
 	//It's probably not totally correct for carts in "SL" mode.
 
 #define PPUT_MMC5
@@ -1202,7 +1226,7 @@ static void RefreshLine(int lastpixel)
 				}
 				else
 				{
-#include "pputile.inc"	    
+#include "pputile.inc"
 				}
 				tochange--;
 			}
@@ -1262,16 +1286,16 @@ static void RefreshLine(int lastpixel)
 #undef RefreshAddr
 
 	//Reverse changes made before.
-	Pal[0]&=63; 
+	Pal[0]&=63;
 	Pal[4]&=63;
 	Pal[8]&=63;
 	Pal[0xC]&=63;
 
 	RefreshAddr=smorkus;
-	if(firsttile<=2 && 2<lasttile && !(PPU[1]&2)) 
+	if(firsttile<=2 && 2<lasttile && !(PPU[1]&2))
 	{
 		uint32 tem;
-		tem=Pal[0]|(Pal[0]<<8)|(Pal[0]<<16)|(Pal[0]<<24);  
+		tem=Pal[0]|(Pal[0]<<8)|(Pal[0]<<16)|(Pal[0]<<24);
 		tem|=0x40404040;
 		*(uint32 *)Plinef=*(uint32 *)(Plinef+4)=tem;
 	}
@@ -1304,8 +1328,8 @@ static void RefreshLine(int lastpixel)
 	//CheckSpriteHit(lasttile*8); //lasttile*8); //lastpixel);
 
 	//This only works right because of a hack earlier in this function.
-	CheckSpriteHit(lastpixel);  
-	
+	CheckSpriteHit(lastpixel);
+
 	if((lastpixel-16)>=0)
 	{
 		InputScanlineHook(Plinef,spork?sprlinebuf:0,linestartts,lasttile*8-16);
@@ -1333,25 +1357,19 @@ static void Fixit1(void)
 	{
 		uint32 rad=RefreshAddr;
 
-		if((rad&0x7000)==0x7000)
+		if((rad & 0x7000) == 0x7000)
 		{
-			rad^=0x7000;
-			if((rad&0x3E0)==0x3A0)
-			{
-				rad^=0x3A0;
-				rad^=0x800;
-			}
+			rad ^= 0x7000;
+			if((rad & 0x3E0) == 0x3A0)
+				rad ^= 0xBA0;
+			else if((rad & 0x3E0) == 0x3e0)
+				rad ^= 0x3e0;
 			else
-			{
-				if((rad&0x3E0)==0x3e0)
-					rad^=0x3e0;
-				else rad+=0x20;
-			}
+				rad += 0x20;
 		}
 		else
-			rad+=0x1000;
-		RefreshAddr=rad;
-		//PPU_hook(RefreshAddr); //,-1);
+			rad += 0x1000;
+		RefreshAddr = rad;
 	}
 }
 
@@ -1591,9 +1609,9 @@ static void FetchSpriteData(void)
 		}
 		//if(ns>=7)
 		//printf("%d %d\n",scanline,ns);
-		
-		//Handle case when >8 sprites per scanline option is enabled. 
-		if(ns>8) PPU_status|=0x20;  
+
+		//Handle case when >8 sprites per scanline option is enabled.
+		if(ns>8) PPU_status|=0x20;
 		else if(PPU_hook)
 		{
 			for(n=0;n<(8-ns);n++)
@@ -1645,13 +1663,13 @@ static void RefreshSprites(void)
 					((J>>1)&0x08) |
 					((J>>3)&0x04) |
 					((J>>5)&0x02) |
-					((J>>7)&0x01);                                          
+					((J>>7)&0x01);
 			}
 
 			C = sprlinebuf+x;
 			VB = (PALRAM+0x10)+((atr&3)<<2);
 
-			if(atr&SP_BACK) 
+			if(atr&SP_BACK)
 			{
 				if(atr&H_FLIP)
 				{
@@ -1671,9 +1689,9 @@ static void RefreshSprites(void)
 					pixdata>>=4;
 					if(J&0x01) C[0]=VB[pixdata]|0x40;
 				} else  {
-					if(J&0x80) C[0]=VB[pixdata&3]|0x40;   
+					if(J&0x80) C[0]=VB[pixdata&3]|0x40;
 					pixdata>>=4;
-					if(J&0x40) C[1]=VB[pixdata&3]|0x40;   
+					if(J&0x40) C[1]=VB[pixdata&3]|0x40;
 					pixdata>>=4;
 					if(J&0x20) C[2]=VB[pixdata&3]|0x40;
 					pixdata>>=4;
@@ -1690,9 +1708,9 @@ static void RefreshSprites(void)
 			} else {
 				if(atr&H_FLIP)
 				{
-					if(J&0x80) C[7]=VB[pixdata&3];   
+					if(J&0x80) C[7]=VB[pixdata&3];
 					pixdata>>=4;
-					if(J&0x40) C[6]=VB[pixdata&3];   
+					if(J&0x40) C[6]=VB[pixdata&3];
 					pixdata>>=4;
 					if(J&0x20) C[5]=VB[pixdata&3];
 					pixdata>>=4;
@@ -1705,10 +1723,10 @@ static void RefreshSprites(void)
 					if(J&0x02) C[1]=VB[pixdata&3];
 					pixdata>>=4;
 					if(J&0x01) C[0]=VB[pixdata];
-				}else{                 
-					if(J&0x80) C[0]=VB[pixdata&3];   
+				}else{
+					if(J&0x80) C[0]=VB[pixdata&3];
 					pixdata>>=4;
-					if(J&0x40) C[1]=VB[pixdata&3];   
+					if(J&0x40) C[1]=VB[pixdata&3];
 					pixdata>>=4;
 					if(J&0x20) C[2]=VB[pixdata&3];
 					pixdata>>=4;
@@ -1838,7 +1856,7 @@ void PPU_ResetHooks()
 
 void FCEUPPU_Reset(void)
 {
-	VRAMBuffer=PPU[0]=PPU[1]=PPU_status=PPU[3]=0;   
+	VRAMBuffer=PPU[0]=PPU[1]=PPU_status=PPU[3]=0;
 	PPUSPL=0;
 	PPUGenLatch=0;
 	RefreshAddr=TempAddr=0;
@@ -1859,7 +1877,7 @@ void FCEUPPU_Power(void)
 	memset(NTARAM,0x00,0x800);
 	memset(PALRAM,0x00,0x20);
     memset(UPALRAM,0x00,0x03);
-	memset(SPRAM,0x00,0x100); 
+	memset(SPRAM,0x00,0x100);
 	FCEUPPU_Reset();
 
 	for(x=0x2000;x<0x4000;x+=8)
@@ -1902,12 +1920,12 @@ int FCEUPPU_Loop(int skip)
 	{
 		X6502_Run(256+85);
 		PPU_status |= 0x80;
-		
+
 		//Not sure if this is correct.  According to Matt Conte and my own tests, it is.
-		//Timing is probably off, though.  
-		//NOTE:  Not having this here breaks a Super Donkey Kong game. 
-		PPU[3]=PPUSPL=0;       
-							   
+		//Timing is probably off, though.
+		//NOTE:  Not having this here breaks a Super Donkey Kong game.
+		PPU[3]=PPUSPL=0;
+
 		//I need to figure out the true nature and length of this delay.
 		X6502_Run(12);
 		if(GameInfo->type==GIT_NSF)
@@ -1917,7 +1935,7 @@ int FCEUPPU_Loop(int skip)
 			if(VBlankON)
 				TriggerNMI();
 		}
-		X6502_Run((scanlines_per_frame-242)*(256+85)-12); //-12); 
+		X6502_Run((scanlines_per_frame-242)*(256+85)-12); //-12);
 		PPU_status&=0x1f;
 		X6502_Run(256);
 
@@ -1935,9 +1953,9 @@ int FCEUPPU_Loop(int skip)
 			}
 			X6502_Run(85-16);
 			if(ScreenON || SpriteON)
-			{  
-				RefreshAddr=TempAddr;  
-				if(PPU_hook) PPU_hook(RefreshAddr&0x3fff);  
+			{
+				RefreshAddr=TempAddr;
+				if(PPU_hook) PPU_hook(RefreshAddr&0x3fff);
 			}
 
 			//Clean this stuff up later.
@@ -1967,7 +1985,7 @@ int FCEUPPU_Loop(int skip)
 						GameHBIRQHook();
 					if(scanline==y && SpriteON) PPU_status|=0x40;
 					X6502_Run((scanline==239)?85:(256+85));
-				}    
+				}
 			}
 			else if(y<240)
 			{
@@ -1990,11 +2008,11 @@ int FCEUPPU_Loop(int skip)
 				DEBUG(FCEUD_UpdatePPUView(scanline, 1));
 				DoLine();
 			}
-			
+
 			if(MMC5Hack && (ScreenON || SpriteON)) MMC5_hb(scanline);
 			for(x=1,max=0,maxref=0;x<7;x++)
 			{
-				
+
 				if(deempcnt[x]>max)
 				{
 					max=deempcnt[x];
@@ -2008,7 +2026,7 @@ int FCEUPPU_Loop(int skip)
 		}
 	} //else... to if(ppudead)
 
-#ifdef FRAMESKIP   
+#ifdef FRAMESKIP
 	if(skip)
 	{
 		FCEU_PutImageDummy();
@@ -2019,9 +2037,9 @@ int FCEUPPU_Loop(int skip)
 	{
 		//mbg 6/21/08 - tileview is being ripped out since i dont know how long its been since it worked
 		//if(tileview) TileView();
-		FCEU_PutImage();     
+		FCEU_PutImage();
 		return(1);
-	}   
+	}
 }
 
 int (*PPU_MASTER)(int skip) = FCEUPPU_Loop;
@@ -2088,7 +2106,7 @@ SFORMAT FCEU_NEWPPU_STATEINFO[] = {
 
 void FCEUPPU_SaveState(void)
 {
-	TempAddrT=TempAddr;   
+	TempAddrT=TempAddr;
 	RefreshAddrT=RefreshAddr;
 }
 
@@ -2103,7 +2121,7 @@ void runppu(int x) {
 	//pputime+=x;
 	//if(cputodo<200) return;
 
-    ppur.status.cycle = (ppur.status.cycle + x) % 
+    ppur.status.cycle = (ppur.status.cycle + x) %
                            ppur.status.end_cycle;
 
 	X6502_Run(x);
@@ -2122,7 +2140,7 @@ struct BGData {
 
 				RefreshAddr = ppur.get_atread();
 				at = CALL_PPUREAD(RefreshAddr);
-				
+
 				//modify at to get appropriate palette shift
 				if(ppur.vt&2) at >>= 4;
 				if(ppur.ht&2) at >>= 2;
@@ -2134,11 +2152,11 @@ struct BGData {
                 if (PPUON)
                 {
 			        ppur.increment_hsc();
-                    if (ppur.status.cycle == 251) 
+                    if (ppur.status.cycle == 251)
                         ppur.increment_vs();
                 }
                 runppu(1);
-				
+
                 ppur.par = nt;
 				RefreshAddr = ppur.get_ptread();
 				pt[0] = CALL_PPUREAD(RefreshAddr);
@@ -2152,6 +2170,15 @@ struct BGData {
 		Record main[34]; //one at the end is junk, it can never be rendered
 	} bgdata;
 
+static inline int PaletteAdjustPixel(int pixel)
+{
+	if((PPU[1]>>5)==0x7)
+		return (pixel&0x3f)|0xc0;
+	else if(PPU[1]&0xE0)
+		return pixel | 0x40;
+	else
+		return (pixel&0x3F)|0x80;
+}
 
 int framectr=0;
 int FCEUX_PPU_Loop(int skip) {
@@ -2173,35 +2200,35 @@ int FCEUX_PPU_Loop(int skip) {
         --ppudead;
         goto finish;
     }
-   
+
 	{
 		PPU_status |= 0x80;
 		ppuphase = PPUPHASE_VBL;
-		
+
 		//Not sure if this is correct.  According to Matt Conte and my own tests, it is.
-		//Timing is probably off, though.  
-		//NOTE:  Not having this here breaks a Super Donkey Kong game. 
-		PPU[3]=PPUSPL=0;       
+		//Timing is probably off, though.
+		//NOTE:  Not having this here breaks a Super Donkey Kong game.
+		PPU[3]=PPUSPL=0;
 		const int delay = 20; //fceu used 12 here but I couldnt get it to work in marble madness and pirates.
-        
+
         ppur.status.sl = 241; //for sprite reads
-		
+
         runppu(delay); //X6502_Run(12);
 		if(VBlankON) TriggerNMI();
         if (PAL)
             runppu(70*(kLineTime)-delay);
         else
 		    runppu(20*(kLineTime)-delay);
-        
+
 		//this seems to run just before the dummy scanline begins
 		PPU_status = 0;
 		//this early out caused metroid to fail to boot. I am leaving it here as a reminder of what not to do
 		//if(!PPUON) { runppu(kLineTime*242); goto finish; }
 
-		//There are 2 conditions that update all 5 PPU scroll counters with the 
-		//contents of the latches adjacent to them. The first is after a write to 
-		//2006/2. The second, is at the beginning of scanline 20, when the PPU starts 
-		//rendering data for the first time in a frame (this update won't happen if 
+		//There are 2 conditions that update all 5 PPU scroll counters with the
+		//contents of the latches adjacent to them. The first is after a write to
+		//2006/2. The second, is at the beginning of scanline 20, when the PPU starts
+		//rendering data for the first time in a frame (this update won't happen if
 		//all rendering is disabled via 2001.3 and 2001.4).
 
 		//if(PPUON)
@@ -2216,9 +2243,10 @@ int FCEUX_PPU_Loop(int skip) {
 		//int xscroll = ppur.fh;
 		//render 241 scanlines (including 1 dummy at beginning)
 		for(int sl=0;sl<241;sl++) {
-            spr_read.start_scanline();
+			spr_read.start_scanline();
 
-            ppur.status.sl = sl;
+			g_rasterpos = 0;
+			ppur.status.sl = sl;
 
 			const int yp = sl-1;
 			ppuphase = PPUPHASE_BG;
@@ -2229,7 +2257,7 @@ int FCEUX_PPU_Loop(int skip) {
 			}
 
 			if(sl != 0) if(MMC5Hack && PPUON) MMC5_hb(yp);
-			
+
 
 			//twiddle the oam buffers
 			const int scanslot = oamslot^1;
@@ -2237,14 +2265,13 @@ int FCEUX_PPU_Loop(int skip) {
 			oamslot ^= 1;
 
 			oamcount = oamcounts[renderslot];
-			
+
 			//the main scanline rendering loop:
 			//32 times, we will fetch a tile and then render 8 pixels.
 			//two of those tiles were read in the last scanline.
 			for(int xt=0;xt<32;xt++) {
 				bgdata.main[xt+2].Read();
 
-
                 //ok, we're also going to draw here.
 				//unless we're on the first dummy scanline
 				if(sl != 0) {
@@ -2257,14 +2284,14 @@ int FCEUX_PPU_Loop(int skip) {
 					//check all the conditions that can cause things to render in these 8px
 					const bool renderspritenow = SpriteON && rendersprites && (xt>0 || SpriteLeft8);
 					const bool renderbgnow = ScreenON && renderbg && (xt>0 || BGLeft8);
-					for(int xp=0;xp<8;xp++,rasterpos++) {
+					for(int xp=0;xp<8;xp++,rasterpos++,g_rasterpos++) {
 
 						//bg pos is different from raster pos due to its offsetability.
 						//so adjust for that here
 						const int bgpos = rasterpos + ppur.fh;
 						const int bgpx = bgpos&7;
 						const int bgtile = bgpos>>3;
-						
+
 						uint8 pixel=0, pixelcolor;
 
 						//generate the BG data
@@ -2279,7 +2306,7 @@ int FCEUX_PPU_Loop(int skip) {
 						bool havepixel = false;
 						for(int s=0;s<oamcount;s++) {
 							uint8* oam = oams[renderslot][s];
-							int x = oam[3]; 
+							int x = oam[3];
 							if(rasterpos>=x && rasterpos<x+8) {
 								//build the pixel.
 								//fetch the LSB of the patterns
@@ -2321,10 +2348,7 @@ int FCEUX_PPU_Loop(int skip) {
 							}
 						}
 
-						//fceu rendering system requires that this be set 
-						//(so that it knows there is a valid pixel there?)
-						pixelcolor |= 0x80;
-						*ptr++ = pixelcolor;
+						*ptr++ = PaletteAdjustPixel(pixelcolor);
 					}
 				}
 			}
@@ -2334,21 +2358,23 @@ int FCEUX_PPU_Loop(int skip) {
 			oamcount=0;
 			const int spriteHeight = Sprite16?16:8;
 			for(int i=0;i<64;i++) {
+				oams[scanslot][oamcount][7] = 0;
 				uint8* spr = SPRAM+i*4;
 				if(yp >= spr[0] && yp < spr[0]+spriteHeight) {
 					//if we already have maxsprites, then this new one causes an overflow,
 					//set the flag and bail out.
 					if(oamcount >= 8 && PPUON) {
 						PPU_status |= 0x20;
-                        if (maxsprites == 8)
-						    break;
+						if (maxsprites == 8)
+							break;
 					}
 
 					//just copy some bytes into the internal sprite buffer
 					for(int j=0;j<4;j++)
 						oams[scanslot][oamcount][j] = spr[j];
-					
-					//note that we stuff the oam index into [6]. 
+					oams[scanslot][oamcount][7] = 1;
+
+					//note that we stuff the oam index into [6].
 					//i need to turn this into a struct so we can have fewer magic numbers
 					oams[scanslot][oamcount][6] = (uint8)i;
 					oamcount++;
@@ -2357,11 +2383,11 @@ int FCEUX_PPU_Loop(int skip) {
 			oamcounts[scanslot] = oamcount;
 
 			//FV is clocked by the PPU's horizontal blanking impulse, and therefore will increment every scanline.
-			//well, according to (which?) tests, maybe at the end of hblank. 
+			//well, according to (which?) tests, maybe at the end of hblank.
 			//but, according to what it took to get crystalis working, it is at the beginning of hblank.
-            
-            //this is done at cycle 251 
-            //rendering scanline, it doesn't need to be scanline 0, 
+
+            //this is done at cycle 251
+            //rendering scanline, it doesn't need to be scanline 0,
             //because on the first scanline when the increment is 0, the vs_scroll is reloaded.
 			//if(PPUON && sl != 0)
 			//	ppur.increment_vs();
@@ -2369,9 +2395,9 @@ int FCEUX_PPU_Loop(int skip) {
 			//todo - think about clearing oams to a predefined value to force deterministic behavior
 
 			//so.. this is the end of hblank. latch horizontal scroll values
-            //do it cycle at 251 
+            //do it cycle at 251
 			 if(PPUON && sl != 0)
-				ppur.install_h_latches(); 
+				ppur.install_h_latches();
 
 			ppuphase = PPUPHASE_OBJ;
 
@@ -2379,7 +2405,7 @@ int FCEUX_PPU_Loop(int skip) {
 			for(int s=0;s<maxsprites;s++) {
 
 				//if we have hit our eight sprite pattern and we dont have any more sprites, then bail
-				if(s==oamcount && s>=8) 
+				if(s==oamcount && s>=8)
 					break;
 
 				//if this is a real sprite sprite, then it is not above the 8 sprite limit.
@@ -2396,6 +2422,13 @@ int FCEUX_PPU_Loop(int skip) {
 				uint32 patternNumber = oam[1];
 				uint32 patternAddress;
 
+				//create deterministic dummy fetch pattern
+				if(!oam[7])
+				{
+					patternNumber = 0;
+					line = 0;
+				}
+
 				//8x16 sprite handling:
 				if(Sprite16) {
 					uint32 bank = (patternNumber&1)<<12;
@@ -2412,27 +2445,28 @@ int FCEUX_PPU_Loop(int skip) {
 				patternAddress += line&7;
 
 				//garbage nametable fetches
-                //reset the scroll counter, happens at cycle 304
-                if (realSprite)
-                {
-                    if ((sl == 0) && PPUON)
-                    {
-                        if (ppur.status.cycle == 304)
-                        {
-                            runppu(1);
-                            ppur.install_latches();
-                            runppu(1);
-                        }
-                        else
-                            runppu(kFetchTime);
-                    }
-                    else
-                        runppu(kFetchTime);
-                }
-                //Dragon's Lair (Europe version mapper 4) 
-                //does not set SpriteON in the beginning but it does
-                //set the bg on so if using the conditional SpriteON the MMC3 counter
-                //the counter will never count and no IRQs will be fired so use PPUON
+				//reset the scroll counter, happens at cycle 304
+				if (realSprite)
+				{
+					if ((sl == 0) && PPUON)
+					{
+						if (ppur.status.cycle == 304)
+						{
+							runppu(1);
+							ppur.install_latches();
+							runppu(1);
+						}
+						else
+							runppu(kFetchTime);
+					}
+					else
+						runppu(kFetchTime);
+				}
+
+				//Dragon's Lair (Europe version mapper 4)
+				//does not set SpriteON in the beginning but it does
+				//set the bg on so if using the conditional SpriteON the MMC3 counter
+				//the counter will never count and no IRQs will be fired so use PPUON
 				if(((PPU[0]&0x38)!=0x18) && s == 2 && PPUON) { //SpriteON ) {
 					//(The MMC3 scanline counter is based entirely on PPU A12, triggered on rising edges (after the line remains low for a sufficiently long period of time))
 					//http://nesdevwiki.org/wiki/index.php/Nintendo_MMC3
@@ -2470,12 +2504,12 @@ int FCEUX_PPU_Loop(int skip) {
 			for(int xt=0;xt<2;xt++)
 				bgdata.main[xt].Read();
 
-			//I'm unclear of the reason why this particular access to memory is made. 
-			//The nametable address that is accessed 2 times in a row here, is also the 
-			//same nametable address that points to the 3rd tile to be rendered on the 
-			//screen (or basically, the first nametable address that will be accessed when 
+			//I'm unclear of the reason why this particular access to memory is made.
+			//The nametable address that is accessed 2 times in a row here, is also the
+			//same nametable address that points to the 3rd tile to be rendered on the
+			//screen (or basically, the first nametable address that will be accessed when
 			//the PPU is fetching background data on the next scanline).
-			//(not implemented yet) 
+			//(not implemented yet)
 			runppu(kFetchTime);
             if (sl == 0)
             {
@@ -2489,13 +2523,14 @@ int FCEUX_PPU_Loop(int skip) {
                 ppur.status.end_cycle = 341;
 			runppu(kFetchTime);
 
-            //After memory access 170, the PPU simply rests for 4 cycles (or the 
-			//equivelant of half a memory access cycle) before repeating the whole 
-			//pixel/scanline rendering process. If the scanline being rendered is the very 
+            //After memory access 170, the PPU simply rests for 4 cycles (or the
+			//equivelant of half a memory access cycle) before repeating the whole
+			//pixel/scanline rendering process. If the scanline being rendered is the very
 			//first one on every second frame, then this delay simply doesn't exist.
             if (ppur.status.end_cycle == 341)
                 runppu(1);
-        }
+
+        } //scanline loop
 
 		if(MMC5Hack && PPUON) MMC5_hb(240);
 
