@@ -1,6 +1,7 @@
 int FCEU_InitVirtualVideo(void);
 void FCEU_KillVirtualVideo(void);
 int SaveSnapshot(void);
+int SaveSnapshot(char[]);
 uint32 GetScreenPixel(int x, int y, bool usebackup);
 int GetScreenPixelPalette(int x, int y, bool usebackup);
 extern uint8 *XBuf;
@@ -25,3 +26,6 @@ extern struct GUIMESSAGE
 extern GUIMESSAGE subtitleMessage;
 
 void FCEU_DrawNumberRow(uint8 *XBuf, int *nstatus, int cur);
+
+std::string FCEUI_GetSnapshotAsName();
+void FCEUI_SetSnapshotAsName(std::string name);
\ No newline at end of file
